/*
 * arm1.5.y
 *
 * yacc implementation code for ARM
 *
 * Copyright (C) 2005 Shakthi Kannan
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */


%{
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

char * to_lower (char * str)
{
	int i; 
	for (i=0;  str[i]; i++) { 
		if (isalpha(str[i])) 
			str[i] = tolower (str[i]);
	} 
	str[i] = '\0'; 
	return str; 

}

void yyerror (const char *str)
{
	fprintf (stderr, "error: %s\n", str);
}

int yywrap ()
{
	return 1;
}

main ()
{
	yyparse();
}

%}

%token CMP CMN 
%token MOV MVN MRS MSR CPSR SPSR CPSR_fc SPSR_fc MRA MAR ACC0
%token TST TEQ AND EOR ORR BIC
%token NOP
%token NUMBER REGISTER
%token LSL LSR ASR ROR RRX
%token NE EQ CS CC MI PL VS VC HI LS GE LT GT LE S

%union
{
	int number;
	char *string;
}


%token <number> NUMBER
%token <string> REGISTER

%%

commands:
				| commands command
				;

command:
			mov
			|
			mvn
			|
			mrs
			|
			msr
			|
			mra
			|
			mar
			|
			cmp
			|
			cmn
			|
			tst
			|
			teq
			|
			and
			|
			eor
			|
			orr
			|
			bic
			|
			nop
			;

mov:
		mov_cmd registers operand2
		|
		mov_cmd condition registers operand2 
		|
		mov_cmd condition status registers operand2 
		|
		mov_cmd status registers operand2 
		;

mvn:
		mvn_cmd registers operand2
		|
		mvn_cmd condition registers operand2 
		|
		mvn_cmd condition status registers operand2 
		|
		mvn_cmd status registers operand2 
		;

mrs:
		mrs_cmd registers psr_end
		|
		mrs_cmd condition registers psr_end
		;

msr:
		msr_cmd psr_fields registers_end 
		|
		msr_cmd condition psr_fields registers_end 
		|
		msr_cmd psr_fields immediate_number 
		|
		msr_cmd condition psr_fields immediate_number 
		;

mra:
		mra_cmd registers registers acc0_end
		|
		mra_cmd condition registers registers acc0_end
		;

mar:
		mar_cmd acc0_reg registers registers_end
		|
		mar_cmd condition acc0_reg registers registers_end
		;

cmp:
		cmp_cmd registers operand2
		|
		cmp_cmd condition registers operand2
		;

cmn:
		cmn_cmd registers operand2
		|
		cmn_cmd condition registers operand2
		;

tst:
		tst_cmd registers operand2
		|
		tst_cmd condition registers operand2
		;

teq:
		teq_cmd registers operand2
		|
		teq_cmd condition registers operand2
		;

and:
		and_cmd registers registers operand2
		|
		and_cmd condition registers registers operand2 
		|
		and_cmd condition status registers registers operand2 
		|
		and_cmd status registers operand2 
		;

eor:
		eor_cmd registers registers operand2
		|
		eor_cmd condition registers registers operand2 
		|
		eor_cmd condition status registers registers operand2 
		|
		eor_cmd status registers operand2 
		;

orr:
		orr_cmd registers registers operand2
		|
		orr_cmd condition registers registers operand2 
		|
		orr_cmd condition status registers registers operand2 
		|
		orr_cmd status registers operand2 
		;

bic:
		bic_cmd registers registers operand2
		|
		bic_cmd condition registers registers operand2 
		|
		bic_cmd condition status registers registers operand2 
		|
		bic_cmd status registers operand2 
		;


nop:
		nop_cmd


status:
	S
		{
			printf ("s ");
		}
		;

mov_cmd:
	MOV
		{
			printf ("mov ");
		}
		;

mvn_cmd:
	MVN
		{
			printf ("mvn ");
		}
		;

mrs_cmd:
	MRS
		{
			printf ("mrs ");
		}
		;

msr_cmd:
	MSR
		{
			printf ("msr ");
		}
		;

mra_cmd:
	MRA
		{
			printf ("mra ");
		}
		;

mar_cmd:
	MAR
		{
			printf ("mar ");
		}
		;

cmp_cmd:
	CMP
		{
			printf ("cmp ");
		}
		;

cmn_cmd:
	CMN
		{
			printf ("cmn ");
		}
		;

tst_cmd:
	TST
		{
			printf ("tst ");
		}
		;

teq_cmd:
	TEQ
		{
			printf ("teq ");
		}
		;

and_cmd:
	AND
		{
			printf ("and ");
		}
		;

eor_cmd:
	EOR
		{
			printf ("eor ");
		}
		;

orr_cmd:
	ORR
		{
			printf ("orr ");
		}
		;

bic_cmd:
	BIC
		{
			printf ("bic ");
		}
		;

nop_cmd:
	NOP
		{
			printf ("nop \n");
		}
		;

condition:
		eq
		|
		ne
		|
		cs
		|
		cc
		|
		mi
		|
		pl
		|
		vs
		|
		vc
		|
		hi
		|
		ls
		|
		ge
		|
		lt
		|
		gt
		|
		le
		;

registers:
	REGISTER
		{
			printf ("%s ", to_lower($1));
		}
		;

registers_end:
	REGISTER
		{
			printf ("%s \n", to_lower($1));
		}
		;

operand2:
	register_only
	|
	immediate_value	
	|
	register_immediate_shift
	|
	register_register_shift
	|
	register_right_extended
	;

register_only:
	REGISTER
		{
			printf ("%s \n", to_lower($1));
		}
		;

immediate_value:	
	NUMBER
		{
			printf ("%d \n", $1);
		}
		;
		
register_immediate_shift:
	registers lsl_op immediate_number
	|
	registers lsr_op immediate_number
	|
	registers asr_op immediate_number
	|
	registers ror_op immediate_number
	;	

register_register_shift:
	registers lsl_op registers_end
	|
	registers lsr_op registers_end
	|
	registers asr_op registers_end
	|
	registers ror_op registers_end
	;	

register_right_extended:
	registers rrx_op
	;

rrx_op:
	RRX
		{
			printf ("rrx \n");
		}
		;

immediate_number:
	NUMBER
		{
			printf ("%d\n", $1);
		}
		;
			
psr_end:
		cpsr_cmd
		|
		spsr_cmd
		;

psr_fields:
		cpsr_fc_cmd
		|
		spsr_fc_cmd
		;

cpsr_cmd:
	CPSR
		{
			printf ("CPSR \n");
		}
		;

spsr_cmd:
	SPSR
		{
			printf ("SPSR \n");
		}
		;

cpsr_fc_cmd:
	CPSR_fc
		{
			printf ("CPSR_fc ");
		}
		;
		
spsr_fc_cmd:
	SPSR_fc
		{
			printf ("SPSR_fc ");
		}
		;

acc0_reg:
	ACC0
		{
			printf ("acc0 ");
		}
		;

acc0_end:
	ACC0
		{
			printf ("acc0 \n");
		}
		;


lsl_op:
	LSL
		{
			printf ("lsl ");
		}
		;

lsr_op:
	LSR
		{
			printf ("lsr ");
		}
		;

asr_op:
	ASR
		{
			printf ("asr ");
		}
		;

ror_op:
	ROR
		{
			printf ("ror ");
		}
		;

eq:
	EQ
		{
			printf ("eq ");
		}
		;

ne:
	NE
		{
			printf ("ne ");
		}
		;

cs:
	CS
		{
			printf ("cs ");
		}
		;

cc:
	CC
		{
			printf ("cc ");
		}
		;

mi:
	MI
		{
			printf ("mi ");
		}
		;

pl:
	PL
		{
			printf ("pl ");
		}
		;

vs:
	VS
		{
			printf ("vs ");
		}
		;

vc:
	VC
		{
			printf ("vc ");
		}
		;

hi:
	HI
		{
			printf ("hi ");
		}
		;

ls:
	LS
		{
			printf ("ls ");
		}
		;

ge:
	GE
		{
			printf ("ge ");
		}
		;

lt:
	LT
		{
			printf ("lt ");
		}
		;

gt:
	GT
		{
			printf ("gt ");
		}
		;
le:
	LE
		{
			printf ("le ");
		}
		;

%%
