#!/bin/bash
# 
# cmn.sh
#
# Copyright (C) 2005 Shakthi Kannan
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
# 
# To test:
# For <Operand2> = immediate_value maximum size

# Data definitions
REG_SIZE=16
COND_SIZE=17
SHIFT_SIZE=4
CMD="cmn"
COMMA=","
FILENAME=$1

# Input error checking
if [ $# -ne 1 ]; then
	echo "Usage: program.sh filename.s"
	echo "One file argument needs to be passed"
	exit 1
fi

# Register array
reg[0]="r0"
reg[1]="r1"
reg[2]="r2"
reg[3]="r3"
reg[4]="r4"
reg[5]="r5"
reg[6]="r6"
reg[7]="r7"
reg[8]="r8"
reg[9]="r9"
reg[10]="sl"
reg[11]="fp"
reg[12]="ip"
reg[13]="sp"
reg[14]="lr"
reg[15]="pc"

# Condition array
cond_arr[0]="eq"
cond_arr[1]="ne"
cond_arr[2]="cs"
cond_arr[3]="hs"
cond_arr[4]="cc"
cond_arr[5]="lo"
cond_arr[6]="mi"
cond_arr[7]="pl"
cond_arr[8]="vs"
cond_arr[9]="vc"
cond_arr[10]="hi"
cond_arr[11]="ls"
cond_arr[12]="ge"
cond_arr[13]="lt"
cond_arr[14]="gt"
cond_arr[15]="le"
cond_arr[16]="al"

# Shift array for <Operand2> lsl, lsr, asr, ror commands
shift_arr[0]="lsl"
shift_arr[1]="lsr"
shift_arr[2]="asr"
shift_arr[3]="ror"

echo " "
echo -n "Writing ${FILENAME} ... "

# Generate code for assembly
echo ".text" >> $FILENAME
echo " "
echo ".globl _start" >> $FILENAME
echo "_start:" >> $FILENAME
echo " "

# cmp Rn, <Operand2>
# 	where <Operand2> = register_only
echo -n "${CMD} Rn, register_only ..."
for (( rn=0; rn<REG_SIZE; rn++ ))
do
	for (( rm=0; rm<REG_SIZE; rm++ ))
	do
		echo "${CMD} ${reg[rn]}${COMMA} ${reg[rm]}"	>> $FILENAME
	done
done
echo " done"

# cmp {cond} Rn, <Operand2>
# 	where {cond} = eq, ne, cs/hs, cc/lo, mi, pl, vs, vc, hi, ls, ge, lt, gt, le, al
# 	where <Operand2> = register_only
echo -n "${CMD}{cond} Rn, register_only ..."
for (( rn=0; rn<REG_SIZE; rn++ ))
do
	for (( rm=0; rm<REG_SIZE; rm++ ))
	do
		for (( cond=0; cond<COND_SIZE; cond++ ))
		do
			echo "${CMD}${cond_arr[cond]} ${reg[rn]}${COMMA} ${reg[rm]}"	>> $FILENAME
		done
	done
done
echo " done"

# cmp Rn, <Operand2>
# 	where <Operand2> = immediate_number
echo -n "${CMD} Rn, immediate_number ..."
for (( rn=0; rn<REG_SIZE; rn++ ))
do
	echo "${CMD} ${reg[rn]}${COMMA} #31"	>> $FILENAME
done
echo " done"

# cmp {cond} Rn, <Operand2>
# 	where {cond} = eq, ne, cs/hs, cc/lo, mi, pl, vs, vc, hi, ls, ge, lt, gt, le, al
# 	where <Operand2> = immediate_number
echo -n "${CMD}{cond} Rn, immediate_number ..."
for (( rn=0; rn<REG_SIZE; rn++ ))
do
	for (( cond=0; cond<COND_SIZE; cond++ ))
	do
		echo "${CMD}${cond_arr[cond]}  ${reg[rn]}${COMMA} #31"	>> $FILENAME
	done
done
echo " done"

# cmp Rn, <Operand2>
# 	where <Operand2> = register_immediate_shift
#			where register_immediate_shift = registers shift_op immediate_number
#				where shift_op = lsl, lsr, asr, ror
echo -n "${CMD} Rn, registers, shift_op immediate_number ..."
for (( rn=0; rn<REG_SIZE; rn++ ))
do
	for (( rm=0; rm<REG_SIZE; rm++ ))
	do
		for (( shift=0; shift<SHIFT_SIZE; shift++ ))
		do
			echo "${CMD} ${reg[rn]}${COMMA} ${reg[rm]}${COMMA} ${shift_arr[shift]} #31" >> $FILENAME
		done
	done
done
echo " done"

# cmp {cond} Rn, <Operand2>
# 	where {cond} = eq, ne, cs/hs, cc/lo, mi, pl, vs, vc, hi, ls, ge, lt, gt, le, al
# 	where <Operand2> = register_immediate_shift
#			where register_immediate_shift = registers shift_op immediate_number
#				where shift_op = lsl, lsr, asr, ror
echo -n "${CMD}{cond} Rn, registers, shift_op immediate_number ..."
for (( rn=0; rn<REG_SIZE; rn++ ))
do
	for (( rm=0; rm<REG_SIZE; rm++ ))
	do
		for (( shift=0; shift<SHIFT_SIZE; shift++ ))
		do
			for (( cond=0; cond<COND_SIZE; cond++ ))
			do
				echo "${CMD}${cond_arr[cond]} ${reg[rn]}${COMMA} ${reg[rm]}${COMMA} ${shift_arr[shift]} #31" >> $FILENAME
			done
		done
	done
done
echo " done"

# cmp Rn, <Operand2>
# 	where <Operand2> = register_register_shift
#			where register_register_shift = registers shift_op registers
#				where shift_op = lsl, lsr, asr, ror
echo -n "${CMD} Rn, registers, shift_op registers ..."
for (( rn=0; rn<REG_SIZE; rn++ ))
do
	for (( rm=0; rm<REG_SIZE; rm++ ))
	do
		for (( rs=0; rs<REG_SIZE; rs++ ))
		do
			for (( shift=0; shift<SHIFT_SIZE; shift++ ))
			do
				echo "${CMD} ${reg[rn]}${COMMA} ${reg[rm]}${COMMA} ${shift_arr[shift]} ${reg[rs]}" >> $FILENAME
			done
		done
	done
done
echo " done"

# cmp {cond} Rn, <Operand2>
# 	where {cond} = eq, ne, cs/hs, cc/lo, mi, pl, vs, vc, hi, ls, ge, lt, gt, le, al
# 	where <Operand2> = register_register_shift
#			where register_register_shift = registers shift_op registers
#				where shift_op = lsl, lsr, asr, ror
echo -n "${CMD}{cond} Rn, registers, shift_op registers ..."
for (( rn=0; rn<REG_SIZE; rn++ ))
do
	for (( rm=0; rm<REG_SIZE; rm++ ))
	do
		for (( rs=0; rs<REG_SIZE; rs++ ))
		do
			for (( shift=0; shift<SHIFT_SIZE; shift++ ))
			do
				for (( cond=0; cond<COND_SIZE; cond++ ))
				do
					echo "${CMD}${cond_arr[cond]} ${reg[rn]}${COMMA} ${reg[rm]}${COMMA} ${shift_arr[shift]} ${reg[rs]}"	>> $FILENAME
				done
			done
		done
	done
done
echo " done"

# cmp Rn, <Operand2>
# 	where <Operand2> = register_right_extended
#			where register_right_extended = registers rrx_op
echo -n "${CMD} Rn, registers, rrx_op ..."
for (( rn=0; rn<REG_SIZE; rn++ ))
do
	for (( rm=0; rm<REG_SIZE; rm++ ))
	do
		echo "${CMD} ${reg[rn]}${COMMA} ${reg[rm]}${COMMA} rrx" >> $FILENAME
	done
done
echo " done"

# cmp {cond} Rn, <Operand2>
# 	where {cond} = eq, ne, cs/hs, cc/lo, mi, pl, vs, vc, hi, ls, ge, lt, gt, le, al
# 	where <Operand2> = register_right_extended
#			where register_right_extended = registers rrx_op
echo -n "${CMD}{cond} Rn, registers, rrx_op ..."
for (( rn=0; rn<REG_SIZE; rn++ ))
do
	for (( rm=0; rm<REG_SIZE; rm++ ))
	do
		for (( cond=0; cond<COND_SIZE; cond++ ))
		do
			echo "${CMD}${cond_arry[cond]} ${reg[rn]}${COMMA} ${reg[rm]}${COMMA} rrx" >> $FILENAME
		done
	done
done
echo " done"

echo " "
echo "... done"
