#!/bin/bash
# 
# mrs.sh
#
# Copyright (C) 2005 Shakthi Kannan
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

# Data definitions
REG_SIZE=16
COND_SIZE=17
CMD="mrs"
COMMA=","
FILENAME=$1

# Input error checking
if [ $# -ne 1 ]; then
	echo "Usage: program.sh filename.s"
	echo "One file argument needs to be passed"
	exit 1
fi

# Register array
reg[0]="r0"
reg[1]="r1"
reg[2]="r2"
reg[3]="r3"
reg[4]="r4"
reg[5]="r5"
reg[6]="r6"
reg[7]="r7"
reg[8]="r8"
reg[9]="r9"
reg[10]="sl"
reg[11]="fp"
reg[12]="ip"
reg[13]="sp"
reg[14]="lr"
reg[15]="pc"

# Condition array
cond_arr[0]="eq"
cond_arr[1]="ne"
cond_arr[2]="cs"
cond_arr[3]="hs"
cond_arr[4]="cc"
cond_arr[5]="lo"
cond_arr[6]="mi"
cond_arr[7]="pl"
cond_arr[8]="vs"
cond_arr[9]="vc"
cond_arr[10]="hi"
cond_arr[11]="ls"
cond_arr[12]="ge"
cond_arr[13]="lt"
cond_arr[14]="gt"
cond_arr[15]="le"
cond_arr[16]="al"

echo " "
echo -n "Writing ${FILENAME} ... "

# Generate code for assembly
echo ".text" >> $FILENAME
echo " "
echo ".globl _start" >> $FILENAME
echo "_start:" >> $FILENAME
echo " "

# mrs Rd, CPSR
echo -n "${CMD} Rd, CPSR ..."
for (( rd=0; rd<REG_SIZE; rd++ ))
do
	echo "${CMD} ${reg[rd]}${COMMA} CPSR"	>> $FILENAME
done
echo " done"

# mrs{cond} Rd, CPSR
# 	where {cond} = eq, ne, cs/hs, cc/lo, mi, pl, vs, vc, hi, ls, ge, lt, gt, le, al
echo -n "${CMD}{cond} Rd, CPSR ..."
for (( cond=0; cond<COND_SIZE; cond++ ))
do
	for (( rd=0; rd<REG_SIZE; rd++ ))
	do
		echo "${CMD}${cond_arr[cond]} ${reg[rd]}${COMMA} CPSR"	>> $FILENAME
	done
done
echo " done"

# mrs Rd, SPSR
echo -n "${CMD} Rd, SPSR ..."
for (( rd=0; rd<REG_SIZE; rd++ ))
do
	echo "${CMD} ${reg[rd]}${COMMA} SPSR"	>> $FILENAME
done
echo " done"

# mrs{cond} Rd, SPSR
# 	where {cond} = eq, ne, cs/hs, cc/lo, mi, pl, vs, vc, hi, ls, ge, lt, gt, le, al
echo -n "${CMD}{cond} Rd, SPSR ..."
for (( cond=0; cond<COND_SIZE; cond++ ))
do
	for (( rd=0; rd<REG_SIZE; rd++ ))
	do
		echo "${CMD}${cond_arr[cond]} ${reg[rd]}${COMMA} SPSR"	>> $FILENAME
	done
done
echo " done"

echo " "
echo "... done"
