#!/bin/bash
# 
# mul.sh
#
# Copyright (C) 2005 Shakthi Kannan
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
# 
# pc not used in mul

# Data definitions
REG_SIZE=15
COND_SIZE=17
CMD="mul"
COMMA=","
FILENAME=$1

# Input error checking
if [ $# -ne 1 ]; then
	echo "Usage: program.sh filename.s"
	echo "One file argument needs to be passed"
	exit 1
fi

# Register array
reg[0]="r0"
reg[1]="r1"
reg[2]="r2"
reg[3]="r3"
reg[4]="r4"
reg[5]="r5"
reg[6]="r6"
reg[7]="r7"
reg[8]="r8"
reg[9]="r9"
reg[10]="sl"
reg[11]="fp"
reg[12]="ip"
reg[13]="sp"
reg[14]="lr"

# Temp register array
tempreg[0]="r0"
tempreg[1]="r1"
tempreg[2]="r2"
tempreg[3]="r3"
tempreg[4]="r4"
tempreg[5]="r5"
tempreg[6]="r6"
tempreg[7]="r7"
tempreg[8]="r8"
tempreg[9]="r9"
tempreg[10]="sl"
tempreg[11]="fp"
tempreg[12]="ip"
tempreg[13]="sp"
tempreg[14]="lr"

# Condition array
cond_arr[0]="eq"
cond_arr[1]="ne"
cond_arr[2]="cs"
cond_arr[3]="hs"
cond_arr[4]="cc"
cond_arr[5]="lo"
cond_arr[6]="mi"
cond_arr[7]="pl"
cond_arr[8]="vs"
cond_arr[9]="vc"
cond_arr[10]="hi"
cond_arr[11]="ls"
cond_arr[12]="ge"
cond_arr[13]="lt"
cond_arr[14]="gt"
cond_arr[15]="le"
cond_arr[16]="al"

echo " "
echo -n "Writing ${FILENAME} ... "

# Generate code for assembly
echo ".text" >> $FILENAME
echo " "
echo ".globl _start" >> $FILENAME
echo "_start:" >> $FILENAME
echo " "

# mul Rd, Rm, Rs
echo -n "${CMD} Rd, Rm, Rs ..."
for (( rd=0; rd<REG_SIZE; rd++ ))
do
	tempreg[0]=${reg[rd]}
	tempreg[rd]=${reg[0]}
	for (( rm=1; rm<REG_SIZE; rm++ ))
	do
		for (( rs=0; rs<REG_SIZE; rs++ ))
		do
			echo "${CMD} ${reg[rd]}${COMMA} ${tempreg[rm]}${COMMA} ${reg[rs]}"	>> $FILENAME
		done
	done
	tempreg[rd]=${reg[rd]}
	tempreg[0]=${reg[0]}
done
echo " done"

# mul{cond} Rd, Rm, Rs
echo -n "${CMD}{cond} Rd, Rm, Rs ..."
for (( rd=0; rd<REG_SIZE; rd++ ))
do
	tempreg[0]=${reg[rd]}
	tempreg[rd]=${reg[0]}
	for (( rm=1; rm<REG_SIZE; rm++ ))
	do
		for (( rs=0; rs<REG_SIZE; rs++ ))
		do
			for (( cond=0; cond<COND_SIZE; cond++ ))
			do
				echo "${CMD}${cond_arr[cond]} ${reg[rd]}${COMMA} ${tempreg[rm]}${COMMA} ${reg[rs]}"	>> $FILENAME
			done
		done
	done
	tempreg[rd]=${reg[rd]}
	tempreg[0]=${reg[0]}
done
echo " done"

# With status

# mul s Rd, Rm, Rs
echo -n "${CMD}s Rd, Rm, Rs ..."
for (( rd=0; rd<REG_SIZE; rd++ ))
do
	tempreg[0]=${reg[rd]}
	tempreg[rd]=${reg[0]}
	for (( rm=1; rm<REG_SIZE; rm++ ))
	do
		for (( rs=0; rs<REG_SIZE; rs++ ))
		do
			echo "${CMD}s ${reg[rd]}${COMMA} ${tempreg[rm]}${COMMA} ${reg[rs]}"	>> $FILENAME
		done
	done
	tempreg[rd]=${reg[rd]}
	tempreg[0]=${reg[0]}
done
echo " done"

# mul{cond}s Rd, Rm, Rs
echo -n "${CMD}{cond}s Rd, Rm, Rs ..."
for (( rd=0; rd<REG_SIZE; rd++ ))
do
	tempreg[0]=${reg[rd]}
	tempreg[rd]=${reg[0]}
	for (( rm=1; rm<REG_SIZE; rm++ ))
	do
		for (( rs=0; rs<REG_SIZE; rs++ ))
		do
			for (( cond=0; cond<COND_SIZE; cond++ ))
			do
				echo "${CMD}${cond_arr[cond]}s ${reg[rd]}${COMMA} ${tempreg[rm]}${COMMA} ${reg[rs]}"	>> $FILENAME
			done
		done
	done
	tempreg[rd]=${reg[rd]}
	tempreg[0]=${reg[0]}
done
echo " done"

echo " "
echo "... done"
