#!/bin/bash
# 
# umlal.sh
#
# Copyright (C) 2005 Shakthi Kannan
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA


CMD="umlal"
COMMA=","
COND_SIZE=17
SIZE=16
FILENAME=$1

# Input error checking
if [ $# -ne 1 ]; then
	echo "Usage: program.sh filename.s"
	echo "One file argument needs to be passed"
	exit 1
fi

# registers
reg[0]="r0"
reg[1]="r1"
reg[2]="r2"
reg[3]="r3"
reg[4]="r4"
reg[5]="r5"
reg[6]="r6"
reg[7]="r7"
reg[8]="r8"
reg[9]="r9"
reg[10]="sl"
reg[11]="fp"
reg[12]="ip"
reg[13]="sp"
reg[14]="lr"
reg[15]="pc"

# Condition array
cond_arr[0]="eq"
cond_arr[1]="ne"
cond_arr[2]="cs"
cond_arr[3]="hs"
cond_arr[4]="cc"
cond_arr[5]="lo"
cond_arr[6]="mi"
cond_arr[7]="pl"
cond_arr[8]="vs"
cond_arr[9]="vc"
cond_arr[10]="hi"
cond_arr[11]="ls"
cond_arr[12]="ge"
cond_arr[13]="lt"
cond_arr[14]="gt"
cond_arr[15]="le"
cond_arr[16]="al"

# umull RdLo, RdHi, Rm, Rs
for (( rdlo=0; rdlo<SIZE; rdlo++ ))
do
  for (( rdhi=0; rdhi<SIZE; rdhi++ ))
  do
    for (( rm=0; rm<SIZE; rm++ ))
    do
      for (( rs=0; rs<SIZE; rs++ ))
      do
	echo "${reg[rdlo]}${COMMA}${reg[rdhi]}${COMMA}${reg[rm]}${COMMA}${reg[rs]}" >> umull.one
      done
    done
  done
done

awk -F"," ' $1 != $2 && $2 != $3 && $3 != $1 && $2 != "pc"' umull.one >> umull.two
sed "s/^/umull /" umull.two >> $FILENAME
rm -f umull.one umull.two
echo "${CMD} RdLo, RdHi, Rm, Rs ... done"

# With condition

# umull {cond} RdLo, RdHi, Rm, Rs
# 	where {cond} = eq, ne, cs/hs, cc/lo, mi, pl, vs, vc, hi, ls, ge, lt, gt, le, al
for (( cond=0; cond<COND_SIZE; cond++ ))
do
  for (( rdlo=0; rdlo<SIZE; rdlo++ ))
  do
    for (( rdhi=0; rdhi<SIZE; rdhi++ ))
    do
      for (( rm=0; rm<SIZE; rm++ ))
      do
	for (( rs=0; rs<SIZE; rs++ ))
	do
	  echo "${reg[rdlo]}${COMMA}${reg[rdhi]}${COMMA}${reg[rm]}${COMMA}${reg[rs]}" >> umull.one
	done
      done
    done
  done

  awk -F"," ' $1 != $2 && $2 != $3 && $3 != $1 && $2 != "pc"' umull.one >> umull.two
  sed "s/^/umull${cond_arr[cond]} /" umull.two >> $FILENAME
  rm -f umull.one umull.two
done
echo "${CMD}{cond} RdLo, RdHi, Rm, Rs ... done"

# With status

# umulls RdLo, RdHi, Rm, Rs
for (( rdlo=0; rdlo<SIZE; rdlo++ ))
do
  for (( rdhi=0; rdhi<SIZE; rdhi++ ))
  do
    for (( rm=0; rm<SIZE; rm++ ))
    do
      for (( rs=0; rs<SIZE; rs++ ))
      do
	echo "${reg[rdlo]}${COMMA}${reg[rdhi]}${COMMA}${reg[rm]}${COMMA}${reg[rs]}" >> umull.one
      done
    done
  done
done

awk -F"," ' $1 != $2 && $2 != $3 && $3 != $1 && $2 != "pc"' umull.one >> umull.two
sed "s/^/umulls /" umull.two >> $FILENAME
rm -f umull.one umull.two
echo "${CMD}s RdLo, RdHi, Rm, Rs ... done"

# With condition and status

# umull {cond}s RdLo, RdHi, Rm, Rs
# 	where {cond} = eq, ne, cs/hs, cc/lo, mi, pl, vs, vc, hi, ls, ge, lt, gt, le, al
for (( cond=0; cond<COND_SIZE; cond++ ))
do
  for (( rdlo=0; rdlo<SIZE; rdlo++ ))
  do
    for (( rdhi=0; rdhi<SIZE; rdhi++ ))
    do
      for (( rm=0; rm<SIZE; rm++ ))
      do
	for (( rs=0; rs<SIZE; rs++ ))
	do
	  echo "${reg[rdlo]}${COMMA}${reg[rdhi]}${COMMA}${reg[rm]}${COMMA}${reg[rs]}" >> umull.one
	done
      done
    done
  done

  awk -F"," ' $1 != $2 && $2 != $3 && $3 != $1 && $2 != "pc"' umull.one >> umull.two
  sed "s/^/umull${cond_arr[cond]}s /" umull.two >> $FILENAME
  rm -f umull.one umull.two
done
echo "${CMD}{cond}s RdLo, RdHi, Rm, Rs ... done"

